<?php

namespace Drupal\content_reporting\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Database\Query\PagerSelectExtender;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\StreamedResponse;

/**
 * Provides functionality for content reporting.
 *
 * Extends the base Controller class to manage report data and user permissions.
 */
class ContentReportingController extends ControllerBase {
  use StringTranslationTrait;
  /**
   * The current user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a ContentReportingController object.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(AccountProxyInterface $current_user, Connection $database, LoggerInterface $logger, FormBuilderInterface $form_builder, ConfigFactoryInterface $config_factory) {
    $this->currentUser = $current_user;
    $this->database = $database;
    $this->logger = $logger;
    $this->formBuilder = $form_builder;
    $this->configFactory = $config_factory;
  }

  /**
   * Creates an instance of the ContentReportingController.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *   The service container.
   *
   * @return static
   *   The controller instance.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user'),
      $container->get('database'),
      $container->get('logger.factory')->get('content_reporting'),
      $container->get('form_builder'),
      $container->get('config.factory')
    );
  }

  /**
   * Retrieves a report based on the provided filters.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request object containing query parameters.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   A JSON response containing the report data or an error message.
   */
  public function getReport(Request $request) {
    $config = $this->configFactory->get('content_reporting.settings');
    // Check user permissions to ensure they can view the report.
    if (!$this->currentUser->hasPermission('content reporting view')) {
      return new JsonResponse([
        'status' => 'error',
        'message' => 'Access denied',
        // Return 403 if the user does not have permission.
      ], 403);
    }
    $fields = ['id', 'title', 'views', 'nid'];
    // Define table headers with sorting attributes.
    $header = [
      ['data' => 'ID', 'field' => 'id', 'sort' => 'id'],
      ['data' => 'Title', 'field' => 'title', 'sort' => 'title'],
      ['data' => $this->t('Views'), 'field' => 'views', 'sort' => 'views'],
    ];

    if ($config->get('track_clicks')) {
      $header[] = ['data' => $this->t('Clicks')];
    }

    if ($config->get('track_time_spent')) {
      $header[] = ['data' => $this->t('Time Spent')];
    }

    if ($config->get('track_gdpr')) {
      $fields[] = 'gdpr_consent';
      $header[] = ['data' => $this->t('GDPR count')];
    }
    if ($config->get('track_logged_in_users')) {
      $fields[] = 'uid';
      $header[] = ['data' => $this->t('User')];
    }
    // Main query to get report data from the database.
    $query = $this->database->select('content_reporting_reports', 'r')
      // Avoid duplicates.
      ->distinct()
      ->fields('r', $fields)
      ->extend(PagerSelectExtender::class)
      ->limit(10);

    // Get filters from the request query parameters.
    $filters = [
      'title' => $request->query->get('title'),
      'start_date' => $request->query->get('start_date'),
      'end_date' => $request->query->get('end_date'),
    ];
    if ($config->get('track_gdpr')) {
      $filters[] = ['gdpr_consent' => $request->query->get('gdpr_consent')];
      // Filters based on selected GDPR consent ranges.
      if (!empty($filters['gdpr_consent'])) {
        // Add an expression to calculate the GDPR consent percentage.
        $query->addExpression('(r.gdpr_consent / r.views) * 100', 'gdpr_percentage');
        switch ($filters['gdpr_consent']) {
          case '0_cons':
            // No consent (0 consents).
            $query->condition('r.gdpr_consent', 0);
            break;

          case '0_25':
            // 0% to 25% consent.
            $query->having('gdpr_percentage >= :min', [':min' => 0])
              ->having('gdpr_percentage <= :max', [':max' => 25]);
            break;

          case '25_50':
            // 25% to 50% consent.
            $query->having('gdpr_percentage >= :min', [':min' => 25])
              ->having('gdpr_percentage <= :max', [':max' => 50]);
            break;

          case '50_70':
            // 50% to 70% consent.
            $query->having('gdpr_percentage >= :min', [':min' => 50])
              ->having('gdpr_percentage <= :max', [':max' => 70]);
            break;

          case '70_100':
            // 70% to 100% consent.
            $query->having('gdpr_percentage >= :min', [':min' => 70])
              ->having('gdpr_percentage <= :max', [':max' => 100]);
            break;
        }
      }
    }
    // Define sorting options based on the request query parameters.
    $sort = $request->query->get('order') ? strtolower($request->query->get('order')) : 'id';
    $order = $request->query->get('sort') ? strtoupper($request->query->get('sort')) : 'DESC';
    if (!$order) {
      $order = 'DESC';
    }

    // Apply sorting to the query.
    if (!empty($sort) && in_array($sort, ['id', 'title', 'views', 'clicks', 'time_spent', 'gdpr_consent'])) {
      $query->orderBy($sort, $order);
    } else {
      // Default sorting by ID if no valid sorting option is provided.
      $query->orderBy('id', 'ASC');
    }

    // Apply filters to the query based on the request parameters.
    if (!empty($filters['title'])) {
      $query->condition('title', '%' . $filters['title'] . '%', 'LIKE');
    }
    if (!empty($filters['start_date'])) {
      $query->condition('report_date', strtotime($filters['start_date']), '>=');
    }
    if (!empty($filters['end_date'])) {
      $query->condition('report_date', strtotime($filters['end_date']), '<=');
    }

    // Group by nid and uid.
    $query->groupBy('r.nid');
    $query->groupBy('r.uid');
    $query->addExpression('SUM(r.views)', 'total_views');

    // Execute the query and fetch results.
    $results = $query->execute()->fetchAll();

    // Prepare rows for the table based on query results.
    $rows = [];
    foreach ($results as $result) {
      if ($config->get('track_logged_in_users')) {
        // Check if the user exists in the users table.
        $userExists = $this->database->select('users_field_data', 'u')
          ->fields('u', ['uid'])
          ->condition('uid', $result->uid)
          ->execute()
          ->fetchField();

        // User exists in the database.
        if ($userExists) {
          $userData = 'Authenticated';
          // User does not exist, so they are considered anonymous.
        } else {
          $userData = 'Anonymous';
        }
      }

      if ($config->get('track_clicks')) {
        // Query to get all "click" interactions for a specific node ID (nid).
        $clicks_query = $this->database->select('content_reporting_interactions', 'i')
          ->fields('i', ['nid', 'id', 'interaction_type', 'element', 'uid'])
          ->distinct()
          ->condition('i.interaction_type', 'click')
          ->condition('i.nid', $result->nid);

        if ($config->get('track_logged_in_users')) {
          $clicks_query->condition('i.uid', $result->uid);
        }

        $clicks_result = $clicks_query->execute()->fetchAll();

        // Collect and count clicks per element.
        $clicks_data = [];
        foreach ($clicks_result as $click) {
          $element = strlen($click->element) > 5 ? substr($click->element, 0, 255) : $click->element;
          if (!isset($clicks_data[$element])) {
            $clicks_data[$element] = 0;
          }
          $clicks_data[$element]++;
        }

        // Generate HTML table for clicks.
        $clicks_table = '<table><tr><th>Element</th><th>Count</th></tr>';
        foreach ($clicks_data as $element => $count) {
          $clicks_table .= '<tr><td>' . $element . '</td><td>' . $count . '</td></tr>';
        }
        $clicks_table .= '</table>';
      }
      if ($config->get('track_time_spent')) {
        // Get all "time_spent" interactions for a specific node ID (nid).
        $time_spent_query = $this->database->select('content_reporting_interactions', 'i')
          ->fields('i', ['nid', 'duration', 'interaction_type', 'uid'])
          ->distinct()
          ->condition('i.interaction_type', 'time_spent')
          ->condition('i.nid', $result->nid);

        if ($config->get('track_logged_in_users')) {
          $time_spent_query->condition('i.uid', $result->uid);
        }

        $time_spent_result = $time_spent_query->execute()->fetchAll();

        // Sum the "time_spent" in PHP.
        $time_spent = 0;
        foreach ($time_spent_result as $record) {
          $time_spent += $record->duration;
        }
      }
      // Create the row with the base fields.
      $row_data = [
        $result->id,
        $result->title,
        $result->total_views,
      ];

      // Add clicks only if click tracking is enabled.
      if ($config->get('track_clicks')) {
        $row_data[] = $clicks_result !== FALSE && $clicks_data
          ? [
            'data' => Markup::create($clicks_table),
            'class' => ['clicks-column'],
          ]
          : 0;
      }

      // Add time spent only if time tracking is enabled.
      if ($config->get('track_time_spent')) {
        $row_data[] = $this->formatTimeSpentWithDateTime($time_spent);
      }
      // Add gdpr consents only if is enabled.
      if ($config->get('track_gdpr')) {
        $row_data[] = $result->gdpr_consent;
      }
      if ($config->get('track_logged_in_users')) {
        // Add the final columns.
        $row_data[] = $userData;
      }

      // Add the complete row to the table data.
      $rows[] = ['data' => $row_data];
    }

    // Attach library for additional styles.
    $build['#attached']['library'][] = 'content_reporting/content_reporting.styles';

    // Add form elements for filters.
    $build['filters'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['filters-container']],
    ];

    // Optional: Add form and form API to manage filters.
    $build['filters']['filters_form'] = $this->formBuilder->getForm('Drupal\content_reporting\Form\ContentReportingFiltersForm');
    $build['filters']['export_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['export-container']],
    ];
    $build['filters']['export_wrapper']['export_link'] = [
      '#type' => 'link',
      '#title' => $this->t('Export to CSV'),
      '#url' => Url::fromRoute('content_reporting.export_to_csv', [], ['query' => $request->query->all()]),
      '#attributes' => [
        'class' => ['button'],
      ],
    ];

    // Build the table.
    $build['report'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No results found.'),
      '#attributes' => [
        'id' => 'content-reporting-table',
        'class' => ['content-reporting'],
      ],
    ];

    // Add pagination control.
    $build['pager'] = [
      '#type' => 'pager',
    ];

    return $build;
  }

  /**
   * Exports the report data to a CSV file.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request object containing query parameters.
   *
   * @return \Symfony\Component\HttpFoundation\StreamedResponse
   *   A streamed response containing the CSV file.
   */
  public function exportToCsv(Request $request) {
    // Same logic as getReport() to get the filtered data.
    $filters = [
      'title' => $request->query->get('title'),
      'start_date' => $request->query->get('start_date'),
      'end_date' => $request->query->get('end_date'),
      'gdpr_consent' => $request->query->get('gdpr_consent'),
    ];

    $query = $this->database->select('content_reporting_reports', 'r')
      ->fields('r', ['id', 'title', 'views', 'gdpr_consent', 'nid']);

    // Apply filters to the query.
    if (!empty($filters['title'])) {
      $query->condition('title', '%' . $filters['title'] . '%', 'LIKE');
    }
    if (!empty($filters['start_date'])) {
      $query->condition('report_date', strtotime($filters['start_date']), '>=');
    }
    if (!empty($filters['end_date'])) {
      $query->condition('report_date', strtotime($filters['end_date']), '<=');
    }

    // Execute the query and fetch results.
    $results = $query->execute()->fetchAll();

    // Prepare CSV response.
    $response = new StreamedResponse(function () use ($results) {
      $handle = fopen('php://output', 'w+');
      fputcsv($handle, ['ID', 'Title', 'Views', 'GDPR Consent']);

      foreach ($results as $result) {
        fputcsv($handle, [$result->id, $result->title, $result->views, $result->gdpr_consent]);
      }

      fclose($handle);
    });

    // Set headers for the CSV download.
    $response->headers->set('Content-Type', 'text/csv');
    $response->headers->set('Content-Disposition', 'attachment; filename="content_report.csv"');

    return $response;
  }

  /**
   *
   */
  public function formatTimeSpentWithDateTime($milliseconds) {
    // Convert milliseconds to seconds.
    $seconds = floor($milliseconds / 1000);

    // Create a DateTime object starting from '00:00:00'.
    // The '@' tells DateTime to treat it as a Unix timestamp.
    $start_time = new \DateTime('@0');

    // Add the seconds as a DateInterval.
    $start_time->add(new \DateInterval('PT' . $seconds . 'S'));

    // Format it as H:i:s.
    return $start_time->format('H:i:s');
  }
}
