<?php

namespace Drupal\content_reporting\Plugin\QueueWorker;

use Drupal\Core\Batch\BatchBuilder;
use Drupal\Core\Database\Connection;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Queue\QueueFactory;
use Drupal\Core\Queue\QueueWorkerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Processes content tracking tasks.
 *
 * @QueueWorker(
 *   id = "content_reporting_track_queue",
 *   title = @Translation("Content Reporting Track Worker"),
 *   cron = {"time" = 60}
 * )
 */
class ContentReportingTrackWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The queue service.
   *
   * @var \Drupal\Core\Queue\QueueInterface
   */
  protected $queue;

  public function __construct(array $configuration, $plugin_id, $plugin_definition, Connection $database, QueueFactory $queue_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->database = $database;
    // Get the queue.
    $this->queue = $queue_factory->get('content_reporting_track_queue');
  }

  /**
   * Static construct.
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('database'),
      $container->get('queue')
    );
  }

  /**
   * Processes queued items in a batch.
   */
  public function run() {
    $batch = new BatchBuilder();
    $batch->setTitle(t('Processing tracking...'));

    $items = [];
    // Adjust the batch size as needed.
    for ($i = 0; $i < 10; $i++) {
      $item = $this->queue->claimItem();
      if ($item) {
        $items[] = $item;
      } else {
        // Stop if there are no more items.
        break;
      }
    }

    // Add each item to the batch for processing.
    foreach ($items as $item) {
      // Pass data to processItem.
      $batch->addOperation([$this, 'processItem'], [$item]);
      // Remove the item from the queue.
      $this->queue->deleteItem($item);
    }

    // Set the finish callback for the batch.
    $batch->setFinishCallback('content_reporting_track_finished');
    batch_set($batch);
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    //var_dump($data['nid']);die;
    if ($data) {
      $query = $this->database->insert('content_reporting_reports')
        ->fields([
          'nid' => (int) $data['nid'],
          'uid' => (int) $data['uid'],
          'views' => isset($data['views']) ? (int) $data['views'] : 0,
          'interactions' => isset($data['interactions']) ? (int) $data['interactions'] : 0,
          'title' => (string) $data['title'],
          'gdpr_consent' => isset($data['gdpr_consent']) ? (int) $data['gdpr_consent'] : 0,
          'report_date' => (string) $data['report_date'],
        ])
        ->execute();
      return $query;
    }
    return $data;
  }
}
